searchData={"items":[{"type":"behaviour","title":"Synq.Channel","doc":"To start a Synq, you'll want to form a channel that `use`s this module.\n\n```\n  use Synq.Channel, web_module: MyAppWeb, max_version: 100\n```\n\nState Management:\n- Specify which assigns to track (sync) using the :track option\n- All other assigns remain in the ethereal plane (server-only)\n- Changes to tracked assigns are automatically synchronized using JSON patches\n\nExample:\n    use Synq.Channel,\n      web_module: MyAppWeb,\n      track: [:spirits, :energy, :mana]\n\n- `track` optional, defaults to []. The keys in assigns to sync with the client.\n- `message_builder` optional, defaults to `{Synq.MessageBuilder, ignore_keys: [:__meta__]}`. If\nset, should be a tuple whose first element is a module that defines `update_state_message/4` and `new_state_message/3` and\nand second element contains any options. Options are passed as final arg to both functions when invoked.\nSee `Synq.MessageBuilder` for details\n- `max_version` optional, defaults to 1000. This is the maximum version number, after which it will\nreset to 0 and begin incrementing again. Version numbers are used to detect a patch message arriving\nout of order. If such a condition is detected by `phx-live-state` a new copy of state is requested.","ref":"Synq.Channel.html"},{"type":"behaviour","title":"System Events - Synq.Channel","doc":"- `synq:error` - Sent when an error occurs\n- `synq:change` - Sent for full state changes\n- `synq:patch` - Sent for incremental state updates using JSON patches","ref":"Synq.Channel.html#module-system-events"},{"type":"callback","title":"Synq.Channel.authorize/3","doc":"Called from join to authorize the connection. Return `{:ok, socket}` to authorize or\n`{:error, reason}` to deny. Default implementation returns `{:ok, socket}`","ref":"Synq.Channel.html#c:authorize/3"},{"type":"callback","title":"Synq.Channel.before_close/2","doc":"Invoked when the channel process is about to exit.\n\nSee `c:GenServer.terminate/2`.","ref":"Synq.Channel.html#c:before_close/2"},{"type":"callback","title":"Synq.Channel.handle_event/3","doc":"Receives an event from the client.\n\nIf you choose not to reply to the message, you can return back {:noreply, socket}\nand an :ok response will be sent back to confirm the event was received.\n\nIf you choose to respond to the event, you can respond with {:reply, reply, socket}.\nThe reply needs to be in the form of:\n\n- `atom()` - status\n- `{atom(), payload()}` - status with payload\n\nYou can also use one of these return types for convenience:\n\n- {:ok, payload, socket} -> {:reply, {:ok, payload}, socket}\n- {:error, error, socket} -> {:reply, {:error, error}, socket}\n\nAny changes to the track will automatically materialize on the client after\nthe response is sent back.","ref":"Synq.Channel.html#c:handle_event/3"},{"type":"callback","title":"Examples - Synq.Channel.handle_event/3","doc":"No reply:\n\n  def handle_event(\"spirit_movement\", %{\"direction\" => direction}, socket) do\n    socket =\n      socket\n      |> update_spirit_position(direction)\n      |> assign(:spirit_position, new_position)\n    {:noreply, socket}\n  end\n\nReply back with result:\n\n  def handle_event(\"summon_spirit\", %{\"type\" => type}, socket) do\n    case SpirtRegistry.summon(type) do\n      {:ok, spirit} ->\n        socket = assign(socket, :spirits, socket.assigns.spirits ++ [spirit])\n        {:ok, %{spirit: spirit}, socket}\n\n      {:error, error} ->\n      {:error, error}\n    end\n  end","ref":"Synq.Channel.html#c:handle_event/3-examples"},{"type":"callback","title":"Synq.Channel.handle_message/2","doc":"Handle regular Elixir process messages.\nWill send events and sync state based on the returned socket.\n\nSee `c:GenServer.handle_info/2`.","ref":"Synq.Channel.html#c:handle_message/2"},{"type":"callback","title":"Synq.Channel.init/3","doc":"Returns the initial application state. Called just after connection","ref":"Synq.Channel.html#c:init/3"},{"type":"type","title":"Synq.Channel.event_response/0","doc":"Possible responses to send back to `c:handle_event/3`","ref":"Synq.Channel.html#t:event_response/0"},{"type":"type","title":"Synq.Channel.message_builder/0","doc":"","ref":"Synq.Channel.html#t:message_builder/0"},{"type":"type","title":"Synq.Channel.payload/0","doc":"Payload types that can be sent between client and server","ref":"Synq.Channel.html#t:payload/0"},{"type":"type","title":"Synq.Channel.reply/0","doc":"Return value for event reply callbacks.\nCan be:\n- status :: atom()              # e.g., :ok, :error\n- {status :: atom(), response :: payload()}  # e.g., {:ok, %{data: ...}}","ref":"Synq.Channel.html#t:reply/0"},{"type":"type","title":"Synq.Channel.track_map/0","doc":"Represents the tracked state that will be synced with the client","ref":"Synq.Channel.html#t:track_map/0"},{"type":"type","title":"Synq.Channel.version/0","doc":"Version number for state updates, used to detect out-of-order messages","ref":"Synq.Channel.html#t:version/0"},{"type":"protocol","title":"Synq.Encoder","doc":"Allows customization of the representation of a given type\nas the state in a `Synq.Channel`. It is called *before* serialization\nand json diffing.","ref":"Synq.Encoder.html"},{"type":"function","title":"Synq.Encoder.encode/2","doc":"","ref":"Synq.Encoder.html#encode/2"},{"type":"type","title":"Synq.Encoder.t/0","doc":"All the types that implement this protocol.","ref":"Synq.Encoder.html#t:t/0"},{"type":"module","title":"Synq.Error","doc":"Standardized error structure for Synq applications.\n\nCore fields:\n- code: Unique identifier (e.g., \"AUTH001\", \"VAL002\")\n- message: Human-readable description\n- details: Optional context/debugging info","ref":"Synq.Error.html"},{"type":"function","title":"Synq.Error.new/3","doc":"Creates a new error struct.","ref":"Synq.Error.html#new/3"},{"type":"function","title":"Examples - Synq.Error.new/3","doc":"iex> Synq.Error.new(\"AUTH001\", \"Invalid credentials\")\n    %Synq.Error{\n      code: \"AUTH001\",\n      message: \"Invalid credentials\",\n      details: nil,\n      timestamp: ~U[2023-...]\n    }","ref":"Synq.Error.html#new/3-examples"},{"type":"function","title":"Synq.Error.normalize/1","doc":"Converts any error term into a standardized Synq.Error struct.","ref":"Synq.Error.html#normalize/1"},{"type":"type","title":"Synq.Error.t/0","doc":"","ref":"Synq.Error.html#t:t/0"},{"type":"module","title":"Synq.Event","doc":"","ref":"Synq.Event.html"},{"type":"type","title":"Synq.Event.t/0","doc":"Represents a CustomEvent to be returned from a reply and dispatched on the client.\n\nFields:\n* name: becomes the name of the CustomEvent to be dispatched\n* detail: becomes the detail property (payload) of the CustomEvent. Not that because\nevents will be serialized as JSON, everything here must implement `Jason.Encoder`","ref":"Synq.Event.html#t:t/0"},{"type":"module","title":"Synq.MessageBuilder","doc":"This is the default message builder for Synq. It will send [json_patch](https://datatracker.ietf.org/doc/html/rfc6902)\nmessages for state updates. The Elixir terms are compared using the `JSONDiff` library to create the\nJSON patch. Because JSONDiff is not aware of any impls of the JSON Encoder protocol, the patch\nmay or may not match the JSON encoding in certain cases. Best effort has been made to handle common\ncases such ecto schemas and DateTime. To gain further control over this process, you\nmay implement the `Synq.Encoder` protocol which will allow you to define a pre-diff representation.\n\nFor a slower, but potentially more correct approach to the problem, there is a [branch](https://github.com/launchscout/live_state/tree/rust-json-diff) containing\nan alternate json diff implemenation in rust.","ref":"Synq.MessageBuilder.html"},{"type":"function","title":"Synq.MessageBuilder.new_state_message/3","doc":"","ref":"Synq.MessageBuilder.html#new_state_message/3"},{"type":"function","title":"Synq.MessageBuilder.update_state_message/4","doc":"","ref":"Synq.MessageBuilder.html#update_state_message/4"},{"type":"type","title":"Synq.MessageBuilder.message_tuple/0","doc":"","ref":"Synq.MessageBuilder.html#t:message_tuple/0"},{"type":"type","title":"Synq.MessageBuilder.version/0","doc":"","ref":"Synq.MessageBuilder.html#t:version/0"},{"type":"module","title":"Synq.TestHelpers","doc":"","ref":"Synq.TestHelpers.html"},{"type":"macro","title":"Synq.TestHelpers.assert_state_change/1","doc":"Asserts that `synq:change` message is received over a channel matching the specified pattern","ref":"Synq.TestHelpers.html#assert_state_change/1"},{"type":"macro","title":"Synq.TestHelpers.assert_state_patch/1","doc":"Asserts that `synq:patch` message is received over a channel matching the specified pattern","ref":"Synq.TestHelpers.html#assert_state_patch/1"},{"type":"function","title":"Synq.TestHelpers.send_event/3","doc":"Pushes a live state event over the channel","ref":"Synq.TestHelpers.html#send_event/3"},{"type":"extras","title":"Synq","doc":"# Synq\n\n[![CI](https://github.com/guess/synq/actions/workflows/ci.yml/badge.svg)](https://github.com/guess/synq/actions/workflows/ci.yml)\n[![License](https://img.shields.io/hexpm/l/synq.svg)](https://github.com/guess/synq/blob/main/LICENSE.md)\n[![Version](https://img.shields.io/hexpm/v/synq.svg)](https://hex.pm/packages/synq)\n[![Hex Docs](https://img.shields.io/badge/documentation-gray.svg)](https://hexdocs.pm/synq)\n\nSynq is an Elixir library for building servers that power interactive web and mobile applications. It offers an alternative approach to state management and client-server communication, particularly suited for applications that don't rely on server-side HTML rendering.","ref":"readme.html"},{"type":"extras","title":"Key Concepts - Synq","doc":"- **Centralized State Management**: Application state is maintained on the server, reducing complexity in state synchronization.\n- **Event-Driven Architecture**: Clients dispatch events to the server, which handles them and updates the state accordingly.\n- **Real-Time Updates**: The server pushes state changes to clients, facilitating real-time interactivity.\n- **Simplified Client Logic**: Client-side code primarily focuses on rendering state and dispatching events.\n- **Platform Agnostic**: Suitable for web applications and mobile apps that manage their own UI rendering.","ref":"readme.html#key-concepts"},{"type":"extras","title":"How It Works - Synq","doc":"1. Clients send events to the server\n2. The server processes events and updates the application state\n3. Updated state is sent back to clients for rendering\n\nThis approach aims to reduce the complexity often found in managing state across both client and server in traditional web and mobile applications.","ref":"readme.html#how-it-works"},{"type":"extras","title":"Use Cases - Synq","doc":"Synq is particularly well-suited for:\n\n- Mobile applications that handle their own UI rendering\n- Web applications requiring real-time updates\n- Scenarios where a unified backend can serve multiple client types (web, mobile, etc.)","ref":"readme.html#use-cases"},{"type":"extras","title":"Comparison to LiveView - Synq","doc":"While Synq shares similar goals with Phoenix LiveView, it takes a different approach:\n\n- LiveView manages both server logic and view presentation in Elixir, primarily for web applications\n- Synq handles server logic in Elixir but relies on client-side code for rendering, making it adaptable for both web and mobile platforms\n\nThis distinction allows Synq to be used in scenarios where full server-side rendering is not possible or desirable, such as in native mobile applications.","ref":"readme.html#comparison-to-liveview"},{"type":"extras","title":"Installation - Synq","doc":"This package can be installed\nby adding `synq` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:synq, \"~> 0.1\"},\n    {:cors_plug, \"~> 3.0\"}\n  ]\nend\n```\n\nWhile `cors_plug` is not strictly required, you will very likely want it to be able to add to your endpoint so that\nclients cannot connect to your channel.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Synq","doc":"First you need to set up a socket as you would with other normal [Phoenix Channels](https://hexdocs.pm/phoenix/channels.html)\n\n1. On your Endpoint module, set up a socket for your channel:\n\n```elixir\ndefmodule DemoWeb.Endpoint do\n  socket \"/synq\", DemoWeb.Socket\n...\n```\n\n2. Then create the socket module with the topic to listen to:\n\n```elixir\ndefmodule DemoWeb.Socket do\n  use Phoenix.Socket\n\n  channel \"topic\", DemoWeb.Channel\n  @impl true\n  def connect(_params, socket), do: {:ok, socket}\n\n  @impl true\n  def id(_), do: \"random_id\"\nend\n```\n\n3. Create your channel using the `Synq.Channel` behaviour:\n\n```elixir\ndefmodule DemoWeb.Channel do\n  use Synq.Channel, web_module: DemoWeb\n...\n```\n\n4. Define your channel bindings and initial state:\n\n```elixir\ndefmodule DemoWeb.Channel do\n  use Synq.Channel,\n    web_module: DemoWeb,\n    track: [:inventory, :energy] # State that can manifest in the mortal realm\n\n  @impl true\n  def init(_topic, _payload, socket) do\n    {:ok, assign(socket, inventory: [], energy: 100)}\n  end\nend\n```\n\nState must be a map. It will be sent down as JSON, so anything in it\nmust have a `Jason.Encoder` implementation.\n\nState assigned to the socket will only manifest in clients if included in the `track` option.","ref":"readme.html#usage"},{"type":"extras","title":"Events - Synq","doc":"Handle events from clients using `handle_event/3`:\n\n```elixir\n  @impl true\n  def handle_event(\"gather_energy\", %{\"amount\" => amount}, socket) do\n    socket = update(socket, :energy, &(&1 + amount))\n    {:noreply, socket}\n  end\n\n  @impl true\n  def handle_event(\"summon_item\", %{\"item\" => item}, socket) do\n    case ItemRegistry.summon(item) do\n      {:ok, item} ->\n        socket = update(socket, :inventory, &[item | &1])\n        {:reply, {:ok, %{item: item}}, socket}\n\n      {:error, reason} ->\n        {:error, reason}\n    end\n  end\n```\n\n`c:Synq.Channel.handle_event/3` receives the following arguments\n\n- event name\n- payload\n- current\n\nAnd returns a tuple whose last element is the new state. It can also return\none or many events to dispatch on the calling DOM Element:\n\n```elixir\n  def handle_event(\"add_todo_with_one_reply\", todo, %{todos: todos}) do\n    {:reply, %Event{name: \"reply_event\", detail: %{foo: \"bar\"}}, %{todos: [todo | todos]}}\n  end\n\n  def handle_event(\"add_todo_with_two_replies\", todo, %{todos: todos}) do\n    {:reply,\n     [\n       %Event{name: \"reply_event1\", detail: %{foo: \"bar\"}},\n       %Event{name: \"reply_event2\", detail: %{bing: \"baz\"}}\n     ], %{todos: [todo | todos]}}\n  end\n```","ref":"readme.html#events"},{"type":"extras","title":"Documentation - Synq","doc":"- [API Docs](https://hexdocs.pm/synq/) are available in the [hex package](https://hex.pm/packages/synq).","ref":"readme.html#documentation"},{"type":"extras","title":"Inspriation - Synq","doc":"This library was adapted from [live_state](https://github.com/launchscout/live_state)","ref":"readme.html#inspriation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}